@@ -4,7 +4,7 @@ use crate::plan::generational::global::Gen;
 use crate::plan::global::BasePlan;
 use crate::plan::global::CommonPlan;
 use crate::plan::global::GcStatus;
-use crate::plan::immix::gc_work::TraceKind;
+use crate::plan::immix::gc_work::{TRACE_KIND_DEFRAG, TRACE_KIND_FAST};
 use crate::plan::AllocationSemantics;
 use crate::plan::Plan;
 use crate::plan::PlanConstraints;
@@ -66,7 +66,7 @@ impl<VM: VMBinding> Plan for GenImmix<VM> {
         use enum_map::enum_map;
         CopyConfig {
             copy_mapping: enum_map! {
-                CopySemantics::PromoteMature => CopySelector::Immix(0),
+                CopySemantics::PromoteToMature => CopySelector::Immix(0),
                 CopySemantics::Mature => CopySelector::Immix(0),
                 _ => CopySelector::Unused,
             },
@@ -130,13 +130,11 @@ impl<VM: VMBinding> Plan for GenImmix<VM> {
         } else if defrag {
             debug!("Full heap GC Defrag");
             scheduler
-                .schedule_common_work::<GenImmixMatureGCWorkContext<VM, { TraceKind::Defrag }>>(
-                    self,
-                );
+                .schedule_common_work::<GenImmixMatureGCWorkContext<VM, TRACE_KIND_DEFRAG>>(self);
         } else {
             debug!("Full heap GC Fast");
             scheduler
-                .schedule_common_work::<GenImmixMatureGCWorkContext<VM, { TraceKind::Fast }>>(self);
+                .schedule_common_work::<GenImmixMatureGCWorkContext<VM, TRACE_KIND_FAST>>(self);
         }
     }
 
