@@ -1,64 +1,15 @@
-use crate::plan::global::Plan;
 use crate::policy::mallocspace::metadata::is_chunk_mapped;
 use crate::policy::mallocspace::metadata::is_chunk_marked_unsafe;
 use crate::policy::mallocspace::MallocSpace;
-use crate::policy::space::Space;
-use crate::scheduler::gc_work::*;
 use crate::scheduler::{GCWork, GCWorker, WorkBucketStage};
 use crate::util::heap::layout::vm_layout_constants::BYTES_IN_CHUNK;
 use crate::util::Address;
-use crate::util::ObjectReference;
 use crate::vm::VMBinding;
 use crate::MMTK;
-use std::ops::{Deref, DerefMut};
 use std::sync::atomic::Ordering;
 
 use super::MarkSweep;
 
-pub struct MSProcessEdges<VM: VMBinding> {
-    plan: &'static MarkSweep<VM>,
-    base: ProcessEdgesBase<VM>,
-}
-
-impl<VM: VMBinding> ProcessEdgesWork for MSProcessEdges<VM> {
-    type VM = VM;
-
-    const OVERWRITE_REFERENCE: bool = false;
-    fn new(edges: Vec<Address>, roots: bool, mmtk: &'static MMTK<VM>) -> Self {
-        let base = ProcessEdgesBase::new(edges, roots, mmtk);
-        let plan = base.plan().downcast_ref::<MarkSweep<VM>>().unwrap();
-        Self { plan, base }
-    }
-
-    #[inline]
-    fn trace_object(&mut self, object: ObjectReference) -> ObjectReference {
-        if object.is_null() {
-            return object;
-        }
-        trace!("Tracing object {}", object);
-        if self.plan.ms_space().in_space(object) {
-            self.plan.ms_space().trace_object::<Self>(self, object)
-        } else {
-            self.plan.common().trace_object::<Self>(self, object)
-        }
-    }
-}
-
-impl<VM: VMBinding> Deref for MSProcessEdges<VM> {
-    type Target = ProcessEdgesBase<VM>;
-    #[inline]
-    fn deref(&self) -> &Self::Target {
-        &self.base
-    }
-}
-
-impl<VM: VMBinding> DerefMut for MSProcessEdges<VM> {
-    #[inline]
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.base
-    }
-}
-
 /// Simple work packet that just sweeps a single chunk
 pub struct MSSweepChunk<VM: VMBinding> {
     ms: &'static MallocSpace<VM>,
@@ -98,7 +49,7 @@ impl<VM: VMBinding> GCWork<VM> for MSSweepChunks<VM> {
         // non-atomic accesses
         while chunk < end {
             if is_chunk_mapped(chunk) && unsafe { is_chunk_marked_unsafe(chunk) } {
-                work_packets.push(box MSSweepChunk { ms, chunk });
+                work_packets.push(Box::new(MSSweepChunk { ms, chunk }));
             }
 
             chunk += BYTES_IN_CHUNK;
@@ -117,9 +68,10 @@ impl<VM: VMBinding> GCWork<VM> for MSSweepChunks<VM> {
     }
 }
 
+use crate::scheduler::gc_work::SFTProcessEdges;
 pub struct MSGCWorkContext<VM: VMBinding>(std::marker::PhantomData<VM>);
 impl<VM: VMBinding> crate::scheduler::GCWorkContext for MSGCWorkContext<VM> {
     type VM = VM;
     type PlanType = MarkSweep<VM>;
-    type ProcessEdgesWorkType = MSProcessEdges<VM>;
+    type ProcessEdgesWorkType = SFTProcessEdges<Self::VM>;
 }
