@@ -1,8 +1,6 @@
-use crate::mmtk::MMTK;
 use crate::plan::global::BasePlan;
 use crate::plan::global::CommonPlan;
 use crate::plan::global::GcStatus;
-use crate::plan::global::NoCopy;
 use crate::plan::marksweep::gc_work::{MSGCWorkContext, MSSweepChunks};
 use crate::plan::marksweep::mutator::ALLOCATOR_MAPPING;
 use crate::plan::AllocationSemantics;
@@ -44,13 +42,8 @@ pub const MS_CONSTRAINTS: PlanConstraints = PlanConstraints {
 impl<VM: VMBinding> Plan for MarkSweep<VM> {
     type VM = VM;
 
-    fn gc_init(
-        &mut self,
-        heap_size: usize,
-        vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<VM>>,
-    ) {
-        self.common.gc_init(heap_size, vm_map, scheduler);
+    fn gc_init(&mut self, heap_size: usize, vm_map: &'static VMMap) {
+        self.common.gc_init(heap_size, vm_map);
     }
 
     fn schedule_collection(&'static self, scheduler: &GCWorkScheduler<VM>) {
@@ -78,12 +71,8 @@ impl<VM: VMBinding> Plan for MarkSweep<VM> {
         self.base().collection_required(self, space_full, space)
     }
 
-    fn get_collection_reserve(&self) -> usize {
-        0
-    }
-
-    fn get_pages_used(&self) -> usize {
-        self.common.get_pages_used() + self.ms.reserved_pages()
+    fn get_used_pages(&self) -> usize {
+        self.common.get_used_pages() + self.ms.reserved_pages()
     }
 
     fn base(&self) -> &BasePlan<VM> {
@@ -97,16 +86,6 @@ impl<VM: VMBinding> Plan for MarkSweep<VM> {
     fn constraints(&self) -> &'static PlanConstraints {
         &MS_CONSTRAINTS
     }
-
-    fn create_worker_local(
-        &self,
-        tls: VMWorkerThread,
-        mmtk: &'static MMTK<Self::VM>,
-    ) -> GCWorkerLocalPtr {
-        let mut c = NoCopy::new(mmtk);
-        c.init(tls);
-        GCWorkerLocalPtr::new(c)
-    }
 }
 
 impl<VM: VMBinding> MarkSweep<VM> {
