@@ -6,14 +6,18 @@ use super::{
 };
 use crate::plan::ObjectsClosure;
 use crate::policy::space::SpaceOptions;
+use crate::policy::space::*;
 use crate::policy::space::{CommonSpace, Space, SFT};
 use crate::util::copy::*;
 use crate::util::heap::layout::heap_layout::{Mmapper, VMMap};
 use crate::util::heap::HeapMeta;
 use crate::util::heap::PageResource;
 use crate::util::heap::VMRequest;
+use crate::util::linear_scan::{Region, RegionIterator};
 use crate::util::metadata::side_metadata::{self, *};
-use crate::util::metadata::{self, compare_exchange_metadata, load_metadata, MetadataSpec};
+use crate::util::metadata::{
+    self, compare_exchange_metadata, load_metadata, store_metadata, MetadataSpec,
+};
 use crate::util::object_forwarding as ForwardingWord;
 use crate::util::{Address, ObjectReference};
 use crate::vm::*;
@@ -27,11 +31,7 @@ use crate::{
     MMTK,
 };
 use atomic::Ordering;
-use std::{
-    iter::Step,
-    ops::Range,
-    sync::{atomic::AtomicU8, Arc},
-};
+use std::sync::{atomic::AtomicU8, Arc};
 
 pub struct ImmixSpace<VM: VMBinding> {
     common: CommonSpace<VM>,
@@ -77,6 +77,15 @@ impl<VM: VMBinding> SFT for ImmixSpace<VM> {
         #[cfg(feature = "global_alloc_bit")]
         crate::util::alloc_bit::set_alloc_bit(_object);
     }
+    #[inline(always)]
+    fn sft_trace_object(
+        &self,
+        _trace: SFTProcessEdgesMutRef,
+        _object: ObjectReference,
+        _worker: GCWorkerMutRef,
+    ) -> ObjectReference {
+        panic!("We do not use SFT to trace objects for Immix. sft_trace_object() cannot be used.")
+    }
 }
 
 impl<VM: VMBinding> Space<VM> for ImmixSpace<VM> {
@@ -99,6 +108,9 @@ impl<VM: VMBinding> Space<VM> for ImmixSpace<VM> {
     fn release_multiple_pages(&mut self, _start: Address) {
         panic!("immixspace only releases pages enmasse")
     }
+    fn set_copy_for_sft_trace(&mut self, _semantics: Option<CopySemantics>) {
+        panic!("We do not use SFT to trace objects for Immix. set_copy_context() cannot be used.")
+    }
 }
 
 impl<VM: VMBinding> ImmixSpace<VM> {
@@ -222,9 +234,8 @@ impl<VM: VMBinding> ImmixSpace<VM> {
         let threshold = self.defrag.defrag_spill_threshold.load(Ordering::Acquire);
         // # Safety: ImmixSpace reference is always valid within this collection cycle.
         let space = unsafe { &*(self as *const Self) };
-        let work_packets = self
-            .chunk_map
-            .generate_tasks(|chunk| box PrepareBlockState {
+        let work_packets = self.chunk_map.generate_tasks(|chunk| {
+            Box::new(PrepareBlockState {
                 space,
                 chunk,
                 defrag_threshold: if space.in_defrag() {
@@ -232,7 +243,8 @@ impl<VM: VMBinding> ImmixSpace<VM> {
                 } else {
                     None
                 },
-            });
+            })
+        });
         self.scheduler().work_buckets[WorkBucketStage::Prepare].bulk_add(work_packets);
         // Update line mark state
         if !super::BLOCK_ONLY {
@@ -336,6 +348,7 @@ impl<VM: VMBinding> ImmixSpace<VM> {
             object
         );
         if Block::containing::<VM>(object).is_defrag_source() {
+            debug_assert!(self.in_defrag());
             self.trace_object_with_opportunistic_copy(trace, object, semantics, worker)
         } else {
             self.trace_object_without_moving(trace, object)
@@ -393,14 +406,12 @@ impl<VM: VMBinding> ImmixSpace<VM> {
                 crate::util::alloc_bit::unset_alloc_bit(object);
                 ForwardingWord::forward_object::<VM>(object, semantics, copy_context)
             };
-            if !super::MARK_LINE_AT_SCAN_TIME {
-                self.mark_lines(new_object);
-            }
             debug_assert_eq!(
                 Block::containing::<VM>(new_object).get_state(),
                 BlockState::Marked
             );
             trace.process_node(new_object);
+            debug_assert!(new_object.is_live());
             new_object
         }
     }
@@ -464,11 +475,12 @@ impl<VM: VMBinding> ImmixSpace<VM> {
     /// Hole searching.
     ///
     /// Linearly scan lines in a block to search for the next
-    /// hole, starting from the given line.
+    /// hole, starting from the given line. If we find available lines,
+    /// return a tuple of the start line and the end line (non-inclusive).
     ///
     /// Returns None if the search could not find any more holes.
     #[allow(clippy::assertions_on_constants)]
-    pub fn get_next_available_lines(&self, search_start: Line) -> Option<Range<Line>> {
+    pub fn get_next_available_lines(&self, search_start: Line) -> Option<(Line, Line)> {
         debug_assert!(!super::BLOCK_ONLY);
         let unavail_state = self.line_unavail_state.load(Ordering::Acquire);
         let current_state = self.line_mark_state.load(Ordering::Acquire);
@@ -487,7 +499,7 @@ impl<VM: VMBinding> ImmixSpace<VM> {
         if cursor == mark_data.len() {
             return None;
         }
-        let start = Line::forward(search_start, cursor - start_cursor);
+        let start = search_start.next_nth(cursor - start_cursor);
         // Find limit
         while cursor < mark_data.len() {
             let mark = mark_data.get(cursor);
@@ -496,10 +508,10 @@ impl<VM: VMBinding> ImmixSpace<VM> {
             }
             cursor += 1;
         }
-        let end = Line::forward(search_start, cursor - start_cursor);
-        debug_assert!((start..end)
+        let end = search_start.next_nth(cursor - start_cursor);
+        debug_assert!(RegionIterator::<Line>::new(start, end)
             .all(|line| !line.is_marked(unavail_state) && !line.is_marked(current_state)));
-        Some(start..end)
+        Some((start, end))
     }
 
     pub fn is_last_gc_exhaustive(did_defrag_for_last_gc: bool) -> bool {
@@ -629,12 +641,27 @@ impl<VM: VMBinding> PolicyCopyContext for ImmixCopyContext<VM> {
         align: usize,
         offset: isize,
     ) -> Address {
-        if self.defrag_allocator.immix_space().in_defrag() {
+        if self.get_space().in_defrag() {
             self.defrag_allocator.alloc(bytes, align, offset)
         } else {
             self.copy_allocator.alloc(bytes, align, offset)
         }
     }
+    #[inline(always)]
+    fn post_copy(&mut self, obj: ObjectReference, _bytes: usize) {
+        // Mark the object
+        store_metadata::<VM>(
+            &VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
+            obj,
+            self.get_space().mark_state as usize,
+            None,
+            Some(Ordering::SeqCst),
+        );
+        // Mark the line
+        if !super::MARK_LINE_AT_SCAN_TIME {
+            self.get_space().mark_lines(obj);
+        }
+    }
 }
 
 impl<VM: VMBinding> ImmixCopyContext<VM> {
@@ -648,4 +675,15 @@ impl<VM: VMBinding> ImmixCopyContext<VM> {
             defrag_allocator: ImmixAllocator::new(tls.0, Some(space), plan, true),
         }
     }
+
+    #[inline(always)]
+    fn get_space(&self) -> &ImmixSpace<VM> {
+        // Both copy allocators should point to the same space.
+        debug_assert_eq!(
+            self.defrag_allocator.immix_space().common().descriptor,
+            self.copy_allocator.immix_space().common().descriptor
+        );
+        // Just get the space from either allocator
+        self.defrag_allocator.immix_space()
+    }
 }
