@@ -22,6 +22,7 @@ use std::marker::PhantomData;
 use std::sync::atomic::AtomicU32;
 use std::sync::atomic::{AtomicUsize, Ordering};
 // only used for debugging
+use crate::policy::space::*;
 #[cfg(debug_assertions)]
 use std::collections::HashMap;
 #[cfg(debug_assertions)]
@@ -72,16 +73,37 @@ impl<VM: VMBinding> SFT for MallocSpace<VM> {
     }
 
     // For malloc space, we need to further check the alloc bit.
-    fn is_mmtk_object(&self, object: ObjectReference) -> bool {
+    fn is_in_space(&self, object: ObjectReference) -> bool {
         is_alloced_by_malloc(object)
     }
 
+    /// For malloc space, we just use the side metadata.
+    #[cfg(feature = "is_mmtk_object")]
+    #[inline(always)]
+    fn is_mmtk_object(&self, addr: Address) -> bool {
+        debug_assert!(!addr.is_zero());
+        // `addr` cannot be mapped by us. It should be mapped by the malloc library.
+        debug_assert!(!addr.is_mapped());
+        has_object_alloced_by_malloc(addr)
+    }
+
     fn initialize_object_metadata(&self, object: ObjectReference, _alloc: bool) {
         trace!("initialize_object_metadata for object {}", object);
         let page_addr = conversions::page_align_down(object.to_address());
         set_page_mark(page_addr);
         set_alloc_bit(object);
     }
+
+    #[inline(always)]
+    fn sft_trace_object(
+        &self,
+        trace: SFTProcessEdgesMutRef,
+        object: ObjectReference,
+        _worker: GCWorkerMutRef,
+    ) -> ObjectReference {
+        let trace = trace.into_mut::<VM>();
+        self.trace_object(trace, object)
+    }
 }
 
 impl<VM: VMBinding> Space<VM> for MallocSpace<VM> {
@@ -228,9 +250,15 @@ impl<VM: VMBinding> MallocSpace<VM> {
         address
     }
 
+    pub fn free(&self, addr: Address) {
+        let offset_malloc_bit = is_offset_malloc(addr);
+        let bytes = get_malloc_usable_size(addr, offset_malloc_bit);
+        self.free_internal(addr, bytes, offset_malloc_bit);
+    }
+
     // XXX optimize: We pass the bytes in to free as otherwise there were multiple
     // indirect call instructions in the generated assembly
-    pub fn free(&self, addr: Address, bytes: usize, offset_malloc_bit: bool) {
+    fn free_internal(&self, addr: Address, bytes: usize, offset_malloc_bit: bool) {
         if offset_malloc_bit {
             trace!("Free memory {:x}", addr);
             offset_free(addr);
@@ -361,7 +389,7 @@ impl<VM: VMBinding> MallocSpace<VM> {
             trace!("Object {} has been allocated but not marked", object);
 
             // Free object
-            self.free(obj_start, bytes, offset_malloc);
+            self.free_internal(obj_start, bytes, offset_malloc);
             trace!("free object {}", object);
             unsafe { unset_alloc_bit_unsafe(object) };
 
