@@ -1,4 +1,5 @@
 use super::space::{CommonSpace, Space, SpaceOptions, SFT};
+use crate::policy::space::*;
 use crate::util::alloc::allocator::align_allocation_no_fill;
 use crate::util::constants::LOG_BYTES_IN_WORD;
 use crate::util::heap::layout::heap_layout::{Mmapper, VMMap};
@@ -17,6 +18,9 @@ pub struct MarkCompactSpace<VM: VMBinding> {
 
 const GC_MARK_BIT_MASK: usize = 1;
 
+/// For each MarkCompact object, we need one extra word for storing forwarding pointer (Lisp-2 implementation).
+/// Note that considering the object alignment, we may end up allocating/reserving more than one word per object.
+/// See [`MarkCompactSpace::HEADER_RESERVED_IN_BYTES`].
 pub const GC_EXTRA_HEADER_WORD: usize = 1;
 const GC_EXTRA_HEADER_BYTES: usize = GC_EXTRA_HEADER_WORD << LOG_BYTES_IN_WORD;
 
@@ -26,11 +30,13 @@ impl<VM: VMBinding> SFT for MarkCompactSpace<VM> {
     }
 
     #[inline(always)]
-    fn get_forwarded_object(&self, _object: ObjectReference) -> Option<ObjectReference> {
-        // the current forwarding pointer implementation will store forwarding pointers
-        // in dead objects' header, which is not the case in mark compact. Mark compact
-        // implements its own forwarding mechanism
-        unimplemented!()
+    fn get_forwarded_object(&self, object: ObjectReference) -> Option<ObjectReference> {
+        let forwarding_pointer = Self::get_header_forwarding_pointer(object);
+        if forwarding_pointer.is_null() {
+            None
+        } else {
+            Some(forwarding_pointer)
+        }
     }
 
     fn is_live(&self, object: ObjectReference) -> bool {
@@ -51,6 +57,18 @@ impl<VM: VMBinding> SFT for MarkCompactSpace<VM> {
     fn is_sane(&self) -> bool {
         true
     }
+
+    #[inline(always)]
+    fn sft_trace_object(
+        &self,
+        _trace: SFTProcessEdgesMutRef,
+        _object: ObjectReference,
+        _worker: GCWorkerMutRef,
+    ) -> ObjectReference {
+        // We should not use trace_object for markcompact space.
+        // Depending on which trace it is, we should manually call either trace_mark or trace_forward.
+        panic!("sft_trace_object() cannot be used with mark compact space")
+    }
 }
 
 impl<VM: VMBinding> Space<VM> for MarkCompactSpace<VM> {
@@ -80,6 +98,8 @@ impl<VM: VMBinding> Space<VM> for MarkCompactSpace<VM> {
 }
 
 impl<VM: VMBinding> MarkCompactSpace<VM> {
+    /// We need one extra header word for each object. Considering the alignment requirement, this is
+    /// the actual bytes we need to reserve for each allocation.
     pub const HEADER_RESERVED_IN_BYTES: usize = if VM::MAX_ALIGNMENT > GC_EXTRA_HEADER_BYTES {
         VM::MAX_ALIGNMENT
     } else {
@@ -87,6 +107,46 @@ impl<VM: VMBinding> MarkCompactSpace<VM> {
     }
     .next_power_of_two();
 
+    // The following are a few functions for manipulating header forwarding poiner.
+    // Basically for each allocation request, we allocate extra bytes of [`HEADER_RESERVED_IN_BYTES`].
+    // From the allocation result we get (e.g. `alloc_res`), `alloc_res + HEADER_RESERVED_IN_BYTES` is the cell
+    // address we return to the binding. It ensures we have at least one word (`GC_EXTRA_HEADER_WORD`) before
+    // the cell address, and ensures the cell address is properly aligned.
+    // From the cell address, `cell - GC_EXTRA_HEADER_WORD` is where we store the header forwarding pointer.
+
+    /// Get the address for header forwarding pointer
+    #[inline(always)]
+    fn header_forwarding_pointer_address(object: ObjectReference) -> Address {
+        VM::VMObjectModel::object_start_ref(object) - GC_EXTRA_HEADER_BYTES
+    }
+
+    /// Get header forwarding pointer for an object
+    #[inline(always)]
+    fn get_header_forwarding_pointer(object: ObjectReference) -> ObjectReference {
+        unsafe { Self::header_forwarding_pointer_address(object).load::<ObjectReference>() }
+    }
+
+    /// Store header forwarding pointer for an object
+    #[inline(always)]
+    fn store_header_forwarding_pointer(
+        object: ObjectReference,
+        forwarding_pointer: ObjectReference,
+    ) {
+        unsafe {
+            Self::header_forwarding_pointer_address(object)
+                .store::<ObjectReference>(forwarding_pointer);
+        }
+    }
+
+    // Clear header forwarding pointer for an object
+    #[inline(always)]
+    fn clear_header_forwarding_pointer(object: ObjectReference) {
+        crate::util::memory::zero(
+            Self::header_forwarding_pointer_address(object),
+            GC_EXTRA_HEADER_BYTES,
+        );
+    }
+
     #[allow(clippy::too_many_arguments)]
     pub fn new(
         name: &'static str,
@@ -161,13 +221,7 @@ impl<VM: VMBinding> MarkCompactSpace<VM> {
             trace.process_node(object);
         }
 
-        // For markcompact, only one word is required to store the forwarding pointer
-        // and it is always stored in front of the object start address. However, the
-        // number of extra bytes required is determined by the object alignment
-        let forwarding_pointer =
-            unsafe { (object.to_address() - GC_EXTRA_HEADER_BYTES).load::<Address>() };
-
-        unsafe { (forwarding_pointer + Self::HEADER_RESERVED_IN_BYTES).to_object_reference() }
+        Self::get_header_forwarding_pointer(object)
     }
 
     pub fn test_and_mark(object: ObjectReference) -> bool {
@@ -255,10 +309,24 @@ impl<VM: VMBinding> MarkCompactSpace<VM> {
             let align = VM::VMObjectModel::get_align_when_copied(obj);
             let offset = VM::VMObjectModel::get_align_offset_when_copied(obj);
             to = align_allocation_no_fill::<VM>(to, align, offset);
-            let forwarding_pointer_addr = obj.to_address() - GC_EXTRA_HEADER_BYTES;
-            unsafe { forwarding_pointer_addr.store(to) }
+            let new_obj = VM::VMObjectModel::get_reference_when_copied_to(
+                obj,
+                to + Self::HEADER_RESERVED_IN_BYTES,
+            );
+
+            Self::store_header_forwarding_pointer(obj, new_obj);
+
+            trace!(
+                "Calculate forward: {} (size when copied = {}) ~> {} (size = {})",
+                obj,
+                VM::VMObjectModel::get_size_when_copied(obj),
+                to,
+                copied_size
+            );
+
             to += copied_size;
         }
+        debug!("Calculate forward end: to = {}", to);
     }
 
     pub fn compact(&self) {
@@ -274,28 +342,26 @@ impl<VM: VMBinding> MarkCompactSpace<VM> {
             // clear the alloc bit
             alloc_bit::unset_addr_alloc_bit(obj.to_address());
 
-            let forwarding_pointer_addr = obj.to_address() - GC_EXTRA_HEADER_BYTES;
-            let forwarding_pointer = unsafe { forwarding_pointer_addr.load::<Address>() };
-            if forwarding_pointer != Address::ZERO {
-                let copied_size =
-                    VM::VMObjectModel::get_size_when_copied(obj) + Self::HEADER_RESERVED_IN_BYTES;
-                to = forwarding_pointer;
-                let object_addr = forwarding_pointer + Self::HEADER_RESERVED_IN_BYTES;
-                // clear forwarding pointer
-                crate::util::memory::zero(
-                    forwarding_pointer + Self::HEADER_RESERVED_IN_BYTES - GC_EXTRA_HEADER_BYTES,
-                    GC_EXTRA_HEADER_BYTES,
-                );
-                crate::util::memory::zero(forwarding_pointer_addr, GC_EXTRA_HEADER_BYTES);
+            let forwarding_pointer = Self::get_header_forwarding_pointer(obj);
+
+            trace!("Compact {} to {}", obj, forwarding_pointer);
+            if !forwarding_pointer.is_null() {
+                let copied_size = VM::VMObjectModel::get_size_when_copied(obj);
+                let new_object = forwarding_pointer;
+                Self::clear_header_forwarding_pointer(new_object);
+
                 // copy object
-                let target = unsafe { object_addr.to_object_reference() };
-                VM::VMObjectModel::copy_to(obj, target, Address::ZERO);
+                trace!(" copy from {} to {}", obj, new_object);
+                let end_of_new_object = VM::VMObjectModel::copy_to(obj, new_object, Address::ZERO);
                 // update alloc_bit,
-                alloc_bit::set_alloc_bit(target);
-                to += copied_size
+                alloc_bit::set_alloc_bit(new_object);
+                to = new_object.to_address() + copied_size;
+                debug_assert_eq!(end_of_new_object, to);
             }
         }
 
+        debug!("Compact end: to = {}", to);
+
         // reset the bump pointer
         self.pr.reset_cursor(to);
     }
@@ -306,6 +372,5 @@ impl<VM: VMBinding> crate::util::linear_scan::LinearScanObjectSize for MarkCompa
     #[inline(always)]
     fn size(object: ObjectReference) -> usize {
         VM::VMObjectModel::get_current_size(object)
-            + MarkCompactSpace::<VM>::HEADER_RESERVED_IN_BYTES
     }
 }
