@@ -18,7 +18,9 @@ pub use work_bucket::WorkBucketStage;
 
 mod worker;
 pub use worker::GCWorker;
-pub(crate) use worker::{GCWorkerLocal, GCWorkerLocalPtr};
+
+mod controller;
+pub use controller::GCController;
 
 pub(crate) mod gc_work;
 pub use gc_work::ProcessEdgesWork;
