@@ -111,7 +111,7 @@ impl<VM: VMBinding> WorkBucket<VM> {
     }
     /// Add a work packet to this bucket, with a default priority (1000)
     pub fn add<W: GCWork<VM>>(&self, work: W) {
-        self.add_with_priority(Self::DEFAULT_PRIORITY, box work);
+        self.add_with_priority(Self::DEFAULT_PRIORITY, Box::new(work));
     }
     pub fn bulk_add_with_priority(&self, priority: usize, work_vec: Vec<Box<dyn GCWork<VM>>>) {
         {
@@ -136,7 +136,7 @@ impl<VM: VMBinding> WorkBucket<VM> {
         &mut self,
         pred: impl Fn(&GCWorkScheduler<VM>) -> bool + Send + 'static,
     ) {
-        self.can_open = Some(box pred);
+        self.can_open = Some(Box::new(pred));
     }
     pub fn update(&self, scheduler: &GCWorkScheduler<VM>) -> bool {
         if let Some(can_open) = self.can_open.as_ref() {
@@ -158,6 +158,7 @@ pub enum WorkBucketStage {
     // we may need more than one buckets for each reference strength.
     RefClosure,
     CalculateForwarding,
+    SecondRoots,
     RefForwarding,
     Compact,
     Release,
