@@ -321,6 +321,7 @@ impl Address {
     }
 
     /// returns the chunk index for this address
+    #[inline(always)]
     pub fn chunk_index(self) -> usize {
         use crate::util::conversions;
         conversions::address_to_chunk_index(self)
@@ -499,8 +500,8 @@ impl ObjectReference {
         SFT_MAP.get(Address(self.0)).get_forwarded_object(self)
     }
 
-    pub fn is_mapped(self) -> bool {
-        SFT_MAP.is_in_space(self)
+    pub fn is_in_any_space(self) -> bool {
+        SFT_MAP.is_in_any_space(self)
     }
 
     #[cfg(feature = "sanity")]
