@@ -7,6 +7,6 @@ use std::sync::atomic::Ordering;
 impl VMGlobalLogBitSpec {
     /// Mark the log bit as unlogged (1 means unlogged)
     pub fn mark_as_unlogged<VM: VMBinding>(&self, object: ObjectReference, order: Ordering) {
-        store_metadata::<VM>(&self, object, 1, None, Some(order))
+        store_metadata::<VM>(self, object, 1, None, Some(order))
     }
 }
